package com.ebaiyihui.family.doctor.server.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.doctoruser.api.pojo.base.dto.doctor.QueryPersonnelInfoReq;
import com.doctoruser.api.pojo.base.vo.doctor.PersonnelInfo;
import com.doctoruser.api.pojo.vo.UserInfoByDoctorIdRespVO;
import com.doctoruser.api.pojo.vo.UserInfoByUserIdRespVO;
import com.ebaiyihui.family.doctor.common.bo.Message;
import com.ebaiyihui.family.doctor.common.bo.MessageInfo;
import com.ebaiyihui.family.doctor.common.bo.PatientSignInfo;
import com.ebaiyihui.family.doctor.common.bo.PersonImInformReq;
import com.ebaiyihui.family.doctor.common.vo.*;
import com.ebaiyihui.family.doctor.server.common.constants.IMInformConstants;
import com.ebaiyihui.family.doctor.server.common.constants.ProjProperties;
import com.ebaiyihui.family.doctor.server.entity.PatientEntity;
import com.ebaiyihui.family.doctor.server.entity.PatientSignEntity;
import com.ebaiyihui.family.doctor.server.exception.BusinessException;
import com.ebaiyihui.family.doctor.server.feign.DoctorInfofeignClient;
import com.ebaiyihui.family.doctor.server.feign.IMPushMsgFeignClient;
import com.ebaiyihui.family.doctor.server.feign.UserApiFeignClient;
import com.ebaiyihui.family.doctor.server.mapper.PatientMapper;
import com.ebaiyihui.family.doctor.server.mapper.PatientSignMapper;
import com.ebaiyihui.family.doctor.server.service.PatientMedicalRecordService;
import com.ebaiyihui.family.doctor.server.util.JsonUtil;
import com.ebaiyihui.framework.response.BaseResponse;
import com.ebaiyihui.framework.utils.HttpKit;
import com.ebaiyihui.imforward.client.vo.AccountVO;
import com.ebaiyihui.imforward.client.vo.PushSingleMsgDataVO;
import com.ebaiyihui.imforward.client.vo.PushSingleMsgReqVO;
import com.ebaiyihui.sysinfocloudcommon.vo.medical.MedicalSortReqVo;
import com.ebaiyihui.sysinfocloudcommon.vo.medical.MedicalSortVo;
import com.ebaiyihui.sysinfocloudcommon.vo.medical.PatientDynamicMedicalEntityVo;
import com.ebaiyihui.sysinfocloudcommon.vo.medical.PatientDynamicMedicalIdByAdmIdVo;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.List;


@Service
@Slf4j
public class PatientMedicalRecordServiceImpl implements PatientMedicalRecordService {

    @Autowired
    private PatientSignMapper patientSignMapper;

    @Autowired
    private PatientMapper patientMapper;

    @Autowired
    private DoctorInfofeignClient doctorInfofeignClient;

    @Autowired
    private UserApiFeignClient userApiFeignClient;

    @Autowired
    private IMPushMsgFeignClient imPushMsgFeignClient;

    @Autowired
    private ProjProperties projProperties;

    @Override
    public PatientCaseDetail getMedicalInfoByAdmId(String admId) {
        log.info("根据就诊id查病例详情入参:{} ", admId);
        try {
            PatientDynamicMedicalIdByAdmIdVo patientDynamicMedicalIdByAdmIdVo = new PatientDynamicMedicalIdByAdmIdVo();
            patientDynamicMedicalIdByAdmIdVo.setAdmId(admId);
            log.info("查询病例详情URL:{}, ", projProperties.getMedicalInfoByAdmId());
            String body = HttpKit.jsonPost(projProperties.getMedicalInfoByAdmId(), JSON.toJSONString(patientDynamicMedicalIdByAdmIdVo));
            BaseResponse<PatientDynamicMedicalEntityVo> baseResponse = JSON.parseObject(body, BaseResponse.class);
            if (baseResponse.getData() == null) {
                return null;
            }
            PatientDynamicMedicalEntityVo patientDynamicMedicalEntityVo = JSON.parseObject(JSON.toJSONString(baseResponse.getData()), PatientDynamicMedicalEntityVo.class);
            log.info("根据就诊id查病例详情返回值:{} ", patientDynamicMedicalEntityVo.toString());
            PatientCaseDetail patientCaseDetail = new PatientCaseDetail();
            //重新封装
            BeanUtils.copyProperties(patientDynamicMedicalEntityVo, patientCaseDetail);
            return patientCaseDetail;
        } catch (Exception e) {
            log.error("根据就诊id查病例详情调用失败{}", e.getMessage());
        }
        return null;
    }


    @Override
    public BaseResponse<Object> treatmentCard(String admissionId) {

        PatientSignInfo patientSignInfo = getPatientSignInfo(admissionId);

        List<MessageInfo> messageInfos = new ArrayList<>();
        //自定义-病历
        messageInfos.add(getInformCustomPersonDocParam(patientSignInfo,
                getTreatmentCard(admissionId), IMInformConstants.CASE_DOCTOR_TYPE));
        //患者
        //自定义-病历
        messageInfos.add(getInformCustomPerPatientParam(patientSignInfo,
                getTreatmentCard(admissionId), IMInformConstants.CASE_DOCTOR_TYPE));
        PersonImInformReq parameter = new PersonImInformReq();
        parameter.setMessageInfos(messageInfos);
        parameter.setAdmissionId(admissionId);
        parameter.setBusiCode(IMInformConstants.IM_SYSTEM_BUSINESS_CODE);
        if (pushImInform(messageInfos, admissionId)) {
            log.info("IM INFORM COMPLETE 医生开具复诊病例推送卡片，admissionId:{}", admissionId);
        }
        return BaseResponse.success();
    }

    @Override
    public BaseResponse<Object> getBaseInfoByAdmId(String admId) {
        log.info("根据就诊id查询患者医生基础信息入参admId:{}", admId);
        if (StringUtils.isEmpty(admId)) {
            throw new BusinessException("amdId不能未空");
        }
        BaseInfoByAdmIdVO baseInfoByAdmIdVO = new BaseInfoByAdmIdVO();

        PatientSignEntity patientSignEntity = patientSignMapper.queryByAdmId(admId);
        log.info("patientSignMapper.queryByAdmId返回值:{} ", JSON.toJSONString(patientSignEntity));
        PatientSignInfo patientSignInfo = getPatientSignInfo(admId);
        log.info("getPatientSignInfo返回值:{} ", JSON.toJSONString(patientSignInfo));
        BeanUtils.copyProperties(patientSignInfo, baseInfoByAdmIdVO);
        if (patientSignEntity != null) {
            BeanUtils.copyProperties(patientSignEntity, baseInfoByAdmIdVO);
            baseInfoByAdmIdVO.setDoctorId(String.valueOf(patientSignEntity.getDoctorId()));
            baseInfoByAdmIdVO.setDeptId(String.valueOf(patientSignEntity.getDeptId()));
            baseInfoByAdmIdVO.setOrderId(String.valueOf(patientSignEntity.getId()));
            PatientEntity patientEntity = patientMapper.selectById(patientSignEntity.getPatientId());
            if(patientEntity != null){
                BeanUtils.copyProperties(patientEntity, baseInfoByAdmIdVO);
                baseInfoByAdmIdVO.setPatientGender(String.valueOf(patientEntity.getGender()));
            }
        }
        PatientCaseDetail medicalInfoByAdmId = getMedicalInfoByAdmId(admId);
        if (medicalInfoByAdmId != null && medicalInfoByAdmId.getPrimaryDiagno()!= null) {
            baseInfoByAdmIdVO.setIcdNames(medicalInfoByAdmId.getPrimaryDiagno());
        }
        log.info("根据就诊id查询患者医生基础信息返回值:{} ", JSON.toJSONString(baseInfoByAdmIdVO));
        return BaseResponse.success(baseInfoByAdmIdVO);
    }

    /**
     * 获取消息参数-自定义消息推给医生（个人）
     *
     * @param admissionInfo 就诊信息
     * @param content       推送内容
     * @param <T>
     * @return 不会返回空
     */
    private <T> MessageInfo getInformCustomPersonDocParam(PatientSignInfo admissionInfo, T content,
                                                          String businessCode) {
        String doctorUserId = admissionInfo.getDoctorUserId();
        if (StringUtils.isEmpty(doctorUserId)) {
            log.error("IM INFORM ERROR : 无法查询到接收人信息");
            return new MessageInfo();
        }
        MessageInfo messageInfo = new MessageInfo();
        Message<T> message = new Message<>();
        message.setMessageType(IMInformConstants.SERVICE_SYSTEM_FREE_TYPE);
        message.setBusinessCode(businessCode);
        message.setApplicationCode(IMInformConstants.IM_SYSTEM_BUSINESS_CODE);
        message.setAdmissionId(admissionInfo.getAdmId());
        message.setData(content);

        message.setApplicationCode(IMInformConstants.IM_SYSTEM_BUSINESS_CODE);
        // 医生账号
        messageInfo.setSingleAccount(newAccount(IMInformConstants.DOCTOR_APPLICATION_CODE, doctorUserId));
        // 患者账号
        messageInfo.setSelfAccount(newAccount(IMInformConstants.PATIENT_APPLICATION_CODE,
                admissionInfo.getPatientUserId()));
        messageInfo.setMessage(message);
        return messageInfo;
    }

    private AccountVO newAccount(String appCode, String userId) {
        AccountVO accountVO = new AccountVO();
        accountVO.setAppCode(appCode);
        accountVO.setUserId(userId);
        return accountVO;
    }

    /**
     * 获取消息参数-自定义消息推给病人（个人））
     *
     * @param admissionInfo 就诊信息
     * @param content       推送内容
     * @param <T>
     * @return 不会返回空
     */
    private <T> MessageInfo getInformCustomPerPatientParam(PatientSignInfo admissionInfo, T content,
                                                           String businessCode) {
        String patientUserId = admissionInfo.getPatientUserId();
        if (StringUtils.isEmpty(patientUserId)) {
            log.error("IM INFORM ERROR : 无法查询到接收人信息");
            return new MessageInfo();
        }
        MessageInfo messageInfo = new MessageInfo();
        Message<T> message = new Message<>();
        message.setMessageType(IMInformConstants.SERVICE_SYSTEM_FREE_TYPE);
        message.setBusinessCode(businessCode);
        message.setAdmissionId(admissionInfo.getAdmId());
        message.setApplicationCode(IMInformConstants.IM_SYSTEM_BUSINESS_CODE);
        message.setData(content);
        message.setApplicationCode(IMInformConstants.IM_SYSTEM_BUSINESS_CODE);
        messageInfo.setSingleAccount(newAccount(IMInformConstants.PATIENT_APPLICATION_CODE, patientUserId));
        messageInfo.setSelfAccount(newAccount(IMInformConstants.DOCTOR_APPLICATION_CODE,
                admissionInfo.getDoctorUserId()));
        messageInfo.setMessage(message);
        return messageInfo;
    }

    private TreatmentCard getTreatmentCard(String admissionId) {
        TreatmentCard treatmentCard = new TreatmentCard();
        // todo 病例信息
        PatientCaseDetail medicalInfoByAdmId = getMedicalInfoByAdmId(admissionId);

        if (null == medicalInfoByAdmId) {
            log.info("病例详情查询错误,查询为空");
            return null;
        }

        List<MedicalSortVo> list = new ArrayList<>();

        list = getMedicalUserFills(medicalInfoByAdmId.getServType(), medicalInfoByAdmId.getMedicalDetail(), medicalInfoByAdmId.getHospitalId(), medicalInfoByAdmId.getDeptId());

        log.info("list:{}", JSON.toJSONString(list));

        if (CollectionUtils.isEmpty(list)) {
            return null;
        }
        treatmentCard.setMedicalRecordId(medicalInfoByAdmId.getXId());
        treatmentCard.setMedicalUserFills(list);
        log.info("caseHistory:{}", treatmentCard);
        return treatmentCard;
    }

    /**
     * 获取im推送需要的信息
     * 1.serviceType  2.0 病人账号 2.1 病人名字 3.3 医生生名字  账号 6.拒绝理由 7.订单号
     * 团队的 1.groupId （注意状态，销毁了就不能发） 2.serviceType 3.0 病人用户账号 3.1 病人名字
     * 4.团队的所有医生bean（id，账号，姓名） 5.团队名字 6 拒绝理由 7订单号
     *
     * @param admId
     * @return
     * @description 公共方法
     */
    private PatientSignInfo getPatientSignInfo(String admId) {
        QueryWrapper<PatientSignEntity> wrapper = new QueryWrapper<>();
        PatientSignEntity patientSign = new PatientSignEntity();
        patientSign.setAdmId(admId);
        wrapper.setEntity(patientSign);
        PatientSignEntity patientSignEntity = patientSignMapper.selectOne(wrapper);
        if (patientSignEntity == null) {
            log.error("IM INFORM ERROR : 没有查询到就诊信息，admId:{}", admId);
            return new PatientSignInfo();
        }
        PatientSignInfo patientSignInfo = new PatientSignInfo();
        BeanUtils.copyProperties(patientSignEntity, patientSignInfo);

        patientSignInfo.setSignedId(patientSignEntity.getId());

        // 获取医生基本信息
        QueryPersonnelInfoReq queryPersonnelInfoReq = new QueryPersonnelInfoReq();
        queryPersonnelInfoReq.setDoctorId(String.valueOf(patientSignEntity.getDoctorId()));
        BaseResponse<PersonnelInfo> doctorInfo = doctorInfofeignClient.queryPersonnelInfo(queryPersonnelInfoReq);
        // 调用用户系统查询医生userId
        UserInfoByUserIdRespVO userInfoByUserIdRespVO = queryDocAccountInfo(String.valueOf(patientSignEntity.getDoctorId()));
        if (userInfoByUserIdRespVO == null) {
            log.error("IM INFORM ERROR : 获取医生信息失败 doctorId：{}", String.valueOf(patientSignEntity.getDoctorId()));
            return new PatientSignInfo();
        }
        if (null == doctorInfo && !doctorInfo.isSuccess()) {
            log.error("IM INFORM ERROR : 获取医生信息失败 doctorId：{}", String.valueOf(patientSignEntity.getDoctorId()));
            return new PatientSignInfo();
        }
        patientSignInfo.setDoctorUserId(userInfoByUserIdRespVO.getUserId());
        patientSignInfo.setDoctorHeadPortrait(doctorInfo.getData().getPortrait());
        patientSignInfo.setDoctorProfessional(doctorInfo.getData().getTitle());
        log.info("patientSignInfo:{}" + JSON.toJSONString(patientSignInfo));
        return patientSignInfo;
    }

    private UserInfoByUserIdRespVO queryDocAccountInfo(String doctorId) {

        BaseResponse<UserInfoByDoctorIdRespVO> response = userApiFeignClient.getUserInfoByDoctorId(doctorId);
        log.info("response:{}", JsonUtil.convertObject(response));

        UserInfoByUserIdRespVO userInfoByUserIdRespVO = new UserInfoByUserIdRespVO();
        if (response.isSuccess()) {

            userInfoByUserIdRespVO.setUserId(response.getData().getUserId());
            log.info("UserInfoByUserIdRespVO===>{}", JSON.toJSONString(userInfoByUserIdRespVO));
            return userInfoByUserIdRespVO;
        }

        return null;
    }

    /**
     * Description:封装推送参数
     */
    private boolean pushImInform(List<MessageInfo> messageInfos, String admissionId) {
        boolean flag = true;
        PersonImInformReq parameter = new PersonImInformReq();
        parameter.setMessageInfos(messageInfos);
        parameter.setAdmissionId(admissionId);
        parameter.setBusiCode(IMInformConstants.IM_SYSTEM_BUSINESS_CODE);
        flag = personImInform(parameter);
        return flag;
    }

    /**
     * 个人（单聊）系统消息推送
     *
     * @param param param
     * @return 是否推送成功
     */
    private boolean personImInform(PersonImInformReq param) {
        List<MessageInfo> messageInfos = param.getMessageInfos();
        if (messageInfos == null || messageInfos.isEmpty()) {
            log.error("IM INFORM ERROR : 没有可发送的消息");
            return false;
        }
        List<PushSingleMsgDataVO> msgSingleDatas = new ArrayList<>();
        messageInfos.forEach(e -> {
            PushSingleMsgDataVO msgsingleData = new PushSingleMsgDataVO();
            msgsingleData.setActionType(IMInformConstants.INFORM_SYSTEM_MESSAGE_TYPE);
            msgsingleData.setFromAccount(e.getSelfAccount());
            msgsingleData.setToAccount(e.getSingleAccount());
            msgsingleData.setMessage(JsonUtil.convertObject(e.getMessage()));
            //固定系统消息
            msgSingleDatas.add(msgsingleData);
        });
        PushSingleMsgReqVO parameter = new PushSingleMsgReqVO();
        parameter.setBusiCode(param.getBusiCode());
        parameter.setUniqueId(param.getAdmissionId());
        parameter.setMsgData(msgSingleDatas);
        // 发送IM单人推送
        log.info("推送入参:{}", JSON.toJSONString(parameter));

        boolean pushFlag = pushSingleIm(parameter);
        if (!pushFlag) {
            log.info("IM INFORM ERROR : im 调用im系统推送失败");
            return false;
        }
        return true;
    }

    /**
     * @param reqVO 请求参数
     * @return boolean 发送结果
     * @description: 发送IM单人推送
     */
    private boolean pushSingleIm(PushSingleMsgReqVO reqVO) {
        log.info("调用IM第三方reqVO:{}", JSON.toJSONString(reqVO));
        BaseResponse<?> response = imPushMsgFeignClient.pushSingleMsg(reqVO);
        log.info("发送IM单人推送:{}", response.toString());
        return response.isSuccess();
    }


    public List<MedicalSortVo> getMedicalUserFills(Integer servType, String medicalDetail, String hospitalId, String deptId) {

        try {
            MedicalSortReqVo medicalSortReqVo = new MedicalSortReqVo();
            medicalSortReqVo.setMedicalUserFills(medicalDetail);
            medicalSortReqVo.setHospitalId(hospitalId);
            medicalSortReqVo.setServType(servType);
            if (servType == 5) {
                medicalSortReqVo.setDeptId(deptId);

            }
            log.info("获取患者填写的病例入参==>{}", JSON.toJSONString(medicalSortReqVo));
            String body = HttpKit.jsonPost(projProperties.getGetMedicalUserFills(), JSON.toJSONString(medicalSortReqVo));

            BaseResponse<List<com.ebaiyihui.sysinfocloudcommon.vo.medical.MedicalSortVo>> baseResponse = JSONObject.toJavaObject(JSONObject.parseObject(body), BaseResponse.class);
            List<com.ebaiyihui.sysinfocloudcommon.vo.medical.MedicalSortVo> medicalSortVos = JSON.parseArray(JSON.toJSONString(baseResponse.getData()), com.ebaiyihui.sysinfocloudcommon.vo.medical.MedicalSortVo.class);

            log.info("获取患者填写的病例返回值: " + medicalSortVos.toString());
            return medicalSortVos;
        } catch (Exception e) {
            log.error("获取患者填写的病例调用失败" + e.getMessage());
        }
        return null;
    }
}
